@@ -105,6 +105,40 @@ class SomePipeline(ibllib.pipes.tasks.Pipeline):
         self.tasks = tasks
 
 
+#  job to output a single file (pathlib.Path)
+class GpuTask(ibllib.pipes.tasks.Task):
+    gpu = 1
+
+    def _run(self, overwrite=False):
+        out_files = self.session_path.joinpath('alf', 'gpu.times.npy')
+        out_files.touch()
+        return out_files
+
+
+class TestLocks(unittest.TestCase):
+
+    def test_gpu_lock_and_local_data_handler(self) -> None:
+        with tempfile.TemporaryDirectory() as td:
+            session_path = Path(td).joinpath('algernon', '2021/02/12', '001')
+            session_path.joinpath('alf').mkdir(parents=True)
+            task = GpuTask(session_path, one=None, location='local')
+            assert task.is_locked() is False
+            task.run()
+            assert task.status == 0
+            assert task.is_locked() is False
+            # then make a lock file and make sure it fails and is still locked afterwards
+            task._make_lock_file()
+            task.run()
+            assert task.status == - 2
+            assert task.is_locked()
+            # test the time out feature
+            task.time_out_secs = - 1
+            task._make_lock_file()
+            assert not task.is_locked()
+            task.run()
+            assert task.status == 0
+
+
 class TestPipelineAlyx(unittest.TestCase):
 
     def setUp(self) -> None:
